using System;
using System.Collections;
using Microsoft.ReportingServices.DataProcessing;
using Microsoft.ReportingServices.Interfaces;


namespace Macaw.ReportingServices.SharepointListExtension
{
	/// <summary>
	/// The connection class is the entry point for the extension. Though this class Reporting Services creates the appropriate Command object.
	/// </summary>
	public class Connection : IDbConnection, IDbConnectionExtension, IExtension
	{
		private string _connString;
		private Hashtable _connStringParts = new Hashtable();
		private System.Data.ConnectionState _state = System.Data.ConnectionState.Closed;
		private string _locName = "SharePoint Lists";
		private string _impersonate;
		private bool _integratedSecurity;
		private string _username;
		private string _password;
      
		public Connection()
		{
		}
    
		// Have a constructor that takes a connection string.
		public Connection(string connString)
		{

			ConnectionString = connString;

		}



		/// <summary>
		/// We use the ConnectionString property to indicate the Url of the WSS site we will be querying. To allow for future additions, we follow the
		/// standard DSN syntax of NAME=VALUE;NAME2=VALUE2 The separate name/value pairs are stored in a Hashtable
		/// </summary>
		public string ConnectionString
		{
			get
			{
				return _connString;
			}
			set
			{
				_connString = value;
				string[] parts = _connString.Split(';');
				_connStringParts = new Hashtable();
				foreach(string part in parts)
				{
					if(part.IndexOf("=") < 1)
					{
						throw new ArgumentException(String.Format("The part {0} is not a valid part for a connection string", part));
					}
					_connStringParts.Add(part.Substring(0, part.IndexOf("=")).ToLower(), part.Substring(part.IndexOf("=") + 1));
				}
			}
		}

		/// <summary>
		/// Our extension does not use the ConnectiontimeOut property
		/// </summary>
		public int ConnectionTimeout
		{
			get
			{
				return 0;
			}
			set{}
		}

		/// <summary>
		/// WssSiteUrl accesses the 'site' proeprty from the ConnectionString
		/// </summary>
		public string WssSiteUrl
		{
			get
			{
				if(_connStringParts.ContainsKey("site"))
				{
					string site = (string)_connStringParts["site"];
					if(!site.EndsWith("/")) site = site + "/";
					return site;
				}
				return "";
			}
		}

		/// <summary>
		/// Indicates the state of the connection (Opened, Closed)
		/// </summary>
		public System.Data.ConnectionState State
		{
			get 
			{
				return _state;
			}
		}

		/// <summary>
		/// We don't support Transactions. If we did, we would also have to implement a class
		/// implementing IDbTransaction
		/// </summary>
		/// <returns></returns>
		public IDbTransaction BeginTransaction()
		{
			throw new NotSupportedException();
		}

		/// <summary>
		/// Normally, opening a connection to a datasource is an expensive 
		/// operation. That is why the extension mechanism works with a 
		/// reusable Connection object that can fire multiple commands. 
		/// In our case, the connection is nothing but a container for the 
		/// Url information. We must use the ConnectionState logic, but 
		/// we do nothing but setting the state to Open
		/// </summary>
		public void Open()
		{
			_state = System.Data.ConnectionState.Open;
			return;
		}

		/// <summary>
		/// When Close is called, the Connection should clean up valueable 
		/// resources. In out case, we have nothing to clean up and we just 
		/// set the State to Closed
		/// </summary>
		public void Close()
		{
			_state = System.Data.ConnectionState.Closed;
			return;
		}

		/// <summary>
		/// Reporting Services calls this method to create an instance of the 
		/// right type of IDbCommand. In our case we return a Command object
		/// </summary>
		/// <returns></returns>
		public IDbCommand CreateCommand()
		{
			// Return a new instance of a command object.
			Command result = new Command(this);
			result.CommandTimeout = this.ConnectionTimeout;
			return result;
		}


		public string LocalizedName
		{
			get
			{
				return _locName;
			}
			set
			{
				_locName = value;
			}
		}

		public void SetConfiguration(string configuration)
		{
			// Used to retrieve configuration data from the config file
		}

		public bool IntegratedSecurity
		{
			get
			{
				return _integratedSecurity;
			}
			set
			{
				_integratedSecurity = value;
			}
		}

		public string UserName
		{
			get
			{
				return _username;
			}
			set
			{
				_username = value;
			}
		}

		public string Password
		{
			get
			{
				return _password;
			}
			set
			{
				_password = value;
			}
		}

		public string Impersonate
		{
			get
			{
				return _impersonate;
			}
			set
			{
				_impersonate = value;
			}
		}
      
		public void Dispose() 
		{
		}
	}
}

